from gargantext.util.http     import requires_auth, render, settings
from gargantext.util.db       import session
from gargantext.util.db_cache import cache
from gargantext.models        import Node
from gargantext.constants     import get_resource
from gargantext.constants     import USER_LANG
from .main                    import get_user_params
from datetime                 import datetime


@requires_auth
def ngramtable(request, project_id, corpus_id):
    '''
    Browse and modify all lists together.
       => maplist and mainlist terms in a table
          with groupings as '+' nodes
       => uses API GET batch of lists
       => uses API PUT/DEL for list modifications
       => uses frontend AJAX through Ngrams_dyna_charts_and_table.js
    # TODO refactor Ngrams_dyna_charts_and_table.js
    '''
    # corpus still necessary to find all lists
    corpus = cache.Node[corpus_id]

    # and the project just for project.id in corpusBannerTop
    project = cache.Node[project_id]

    # retrieve all corpora of this user for list import option
    # POSSIBILITY: could do same task in ajax "only if needed"
    #              (use api for that when merged)
    corpora_infos_q = (session.query(Node.id, Node.name)
                              .filter(Node.typename == "CORPUS")
                              .filter(Node.user_id == project.user_id))
                               # .filter(Node.id != corpus_id)
    corpora_infos = corpora_infos_q.all()

    source_type = corpus.resources()[0]['type']
    # rendered page : terms.html
    return render(
        template_name = 'pages/corpora/terms.html',
        request = request,
        context = {
            'debug': settings.DEBUG,
            'user': request.user,
            'date': datetime.now(),
            'project': project,
            'corpus' : corpus,
            'resourcename' : get_resource(source_type)['name'],
            'view': 'terms',

            # for the CSV import modal
            'importroute': "/api/ngramlists/import?onto_corpus=%i"% corpus.id,
            'corporainfos' : corpora_infos,
            #user params
            'user_parameters': get_user_params(request.user),
            'languages': USER_LANG
        },
    )
